
/*
	File:		FontMenuLibrary.h

	Contains:	graphics libraries - font menu library interfaces

	Written by:	Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga

	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	  1/9/95	JD		changed 'boolean' to 'Boolean'
		 <1>	  1/9/95	JD		First checked in.
*/

#ifndef __FONTMENULIBRARY__
#define __FONTMENULIBRARY__

#include <Menus.h>
#include <GXTypes.h>
#include <GXLayout.h>
#include <GXFonts.h>

#ifdef __cplusplus
extern "C" {
#endif

/* for compatibility with old headers */
#define fontMenuLibraryIncludes

enum fontMenuAttributes {
	noInstancesFontMenu = 1
};
typedef long fontMenuAttribute;

typedef Boolean (*fontFilterProc)(gxFont fontID);

void SortMenu(MenuHandle menu);
void DeleteMenuItems(MenuHandle menuH);

long FontMenu(MenuHandle menu);
long FontPlatformMenu(MenuHandle menu, gxFontPlatform platform, gxFontScript script, gxFontLanguage language);
long FontFamilyMenu(MenuHandle menu);
long FontFamilyPlatformMenu(MenuHandle menu, gxFontPlatform platform, gxFontScript script, gxFontLanguage language);
MenuHandle FontStyleMenu(short menuID, gxFont family);
short HierFontMenu(MenuHandle theMenu, short firstHierMenuID, fontFilterProc proc, fontMenuAttribute attr);
gxFont DoHierFontMenuCommand(long menuResult, short hierFontMenuID, long *instanceIndex);
Boolean DoHierFontMenuCommandStyle(long menuResult, short hierFontMenuID, gxStyle aStyle, long matchInfo);
short DoHierFontMenuCommandShape(long menuResult, short hierFontMenuID, gxShape aShape);
long FontToQD(gxFont fontID, long* styleBits);

long GetMenuRunFeatures(MenuHandle menu, gxFont fontID, gxRunFeature feature[]);
void SetMenuRunFeatures(MenuHandle menu, gxFont fontID, long count, const gxRunFeature feature[]);
void FontFeatureMenu(MenuHandle menu, gxFont fontID);
void ToggleFeatureMenuItem(MenuHandle menuH, gxFont fontID, long item);
void ToggleFeatureMenuFeature(MenuHandle menuH, gxFont fontID, gxRunFeature feature);
void StyleFeatureMenu(MenuHandle menuH, const gxStyle style);

long GetDefaultFontFeatures(gxFont fontID, gxRunFeature features[]);
void SetShapeDefaultFeatures(gxShape dst, gxFont fontID);
void SetStyleDefaultFeatures(gxStyle dst, gxFont fontID);

#ifdef __cplusplus
}
#endif

#endif /* __FONTMENULIBRARY__ */
